package com.objis.demohibernate.test;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;

import com.objis.demohibernate.domaine.Formateur;
import com.objis.demohibernate.domaine.Formation;
import com.objis.demohibernate.domaine.FormationInter;
import com.objis.demohibernate.domaine.FormationIntra;
import com.objis.demohibernate.domaine.Lieu;
import com.objis.demohibernate.service.GestionnaireFormation;

public class GestionnaireFormationTest extends TestCase {

	// Tester la cration d'une Formation. Teste cl primaire retourne
	public void testCreerFormation() {
		GestionnaireFormation serviceGestionnaire = new GestionnaireFormation();	

		Lieu lieu1 = new Lieu("Objis","centre d'affaire Regus, 72 rue du Faubourg St Honor , 75008 Paris");
		Lieu lieu2 = new Lieu("So@t","104 bis, rue de Reuilly, 75012 Paris");

		Formateur formateur1 = new Formateur("Mbiandou", "Douglas", 10);
		Formateur formateur2 = new Formateur("Fontan", "Yannick", 10);
		Formateur formateur3 = new Formateur("Dupont", "Jean", 6);
		Formateur formateur4 = new Formateur("Dufour", "Martin", 8);

		Set<Formateur> formateurs = new java.util.HashSet<Formateur>();
		formateurs.add(formateur1);
		formateurs.add(formateur2);
		formateurs.add(formateur3);
		formateurs.add(formateur4);

		// 3 : Utilisation du service
		serviceGestionnaire.creerFormation(new Formation("Hibernate",new Date(),3,lieu2,formateurs));
		serviceGestionnaire.creerFormation(new FormationInter("Struts",new Date(),3,lieu1,formateurs,2));
		serviceGestionnaire.creerFormation(new FormationIntra("Spring",new Date(),3,lieu1,formateurs,"oui"));
		serviceGestionnaire.creerFormation(new FormationIntra("Ajax",new Date(),3,lieu1,formateurs,"oui"));
		serviceGestionnaire.creerFormation(new FormationIntra("Selenium",new Date(),3,lieu2,formateurs,"oui"));

		Formation formation = new Formation("EJB3");	
		Long id = serviceGestionnaire.creerFormation(formation);
		assertNotNull(id);
	}

	// Tester RequeteHqlFormateurs
	public void testRequeteHqlFormateurs() {
		System.out.println("\n############### testRequeteHqlFormateurs ###############");
		List<Formateur> formateurs = null;
		GestionnaireFormation gestionnaire = new GestionnaireFormation();

		formateurs = gestionnaire.requeteHqlFormateurs("from Formateur",null);
		assertTrue(formateurs.size() > 1);

		formateurs = gestionnaire.requeteHqlFormateurs("from Formateur where nom = 'Mbiandou'",null);
		assertTrue(formateurs.size() == 1);

		formateurs = gestionnaire.requeteHqlFormateurs("from Formateur where nom = ?","Mbiandou");
		assertTrue(formateurs.size() == 1);
	}

	// Tester RequeteHqlFormateurs
	public void testRequeteCriteriaFormateurs() {
		System.out.println("\n############### testRequeteHqlFormateurs ###############");
		List<Formateur> formateurs = null;
		GestionnaireFormation gestionnaire = new GestionnaireFormation();

		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.ne("nom", "toto"));
		assertTrue(formateurs.size() > 1);

		formateurs = gestionnaire.requeteCriteriaFormateurs(null);
		assertTrue(formateurs.size() > 1);

		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.like("nom", "Mbiandou"));
		assertTrue(formateurs.size() == 1);

		HashMap<Object, Object> allEq = new HashMap<>();
		allEq.put("anneeExperience", new Integer(10)); allEq.put("nom", "Fontan");
		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.allEq(allEq));
		assertTrue(formateurs.size() == 1);

		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.between("anneeExperience", 7, 9));
		assertTrue(formateurs.size() == 1);

		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.eq("nom", "Dupont"));
		assertTrue(formateurs.size() == 1);

		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.ge("anneeExperience", 10));
		assertTrue(formateurs.size() == 2);

		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.gt("anneeExperience", 8));
		assertTrue(formateurs.size() == 2);

		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.idEq(new Long(4)));
		assertTrue(formateurs.size() == 1);

		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.like("nom", "dou", MatchMode.END));
		assertTrue(formateurs.size() == 1);

		ArrayList<String> in = new ArrayList<String>();
		in.add("Dufour"); in.add("Dupont"); 
		formateurs = gestionnaire.requeteCriteriaFormateurs(Restrictions.in("nom", in));
		assertTrue(formateurs.size() == 2);

		formateurs = gestionnaire.requeteCriteriaFormateurs( Restrictions.or( Restrictions.not(Restrictions.isNotNull("nom")), Restrictions.lt("anneeExperience",8) ));
		assertTrue(formateurs.size() == 1);
	}

	// Tester RequeteHqlFormateurs
	public void testRequeteProjection() {
		System.out.println("\n############### testRequeteProjection ###############");
		GestionnaireFormation gestionnaire = new GestionnaireFormation();
		assertTrue(gestionnaire.requeteProjection());
	}

	// pour ordonner les tests
	public GestionnaireFormationTest(String testName) {
		super(testName);
	}
	public static junit.framework.Test suite() {
		TestSuite suite = new TestSuite("Test GestionnaireFormation");
		suite.addTest(new GestionnaireFormationTest("testCreerFormation"));
		suite.addTest(new GestionnaireFormationTest("testRequeteHqlFormateurs"));
		suite.addTest(new GestionnaireFormationTest("testRequeteCriteriaFormateurs"));
		suite.addTest(new GestionnaireFormationTest("testRequeteProjection"));
		return suite;
	}
}
